// This toolbar procedure file is designed to work with Application
// Base applications.  It includes a class for a toolbar base
// which is defined under the application base.  The toolbars
// are then defined under the toolBar Base.

class ToolBarBase ( appBaseObj )
*----------- 
*- Name.....ToolBar Base (ToolBarBase)
*- Author...Todd Kreuter
*- History..1998.1122 - Original
			// 1999.0923 - No longer pass the large parameter
         // 				Added setLarge method to set button height

*- Notes....This is the base object of an applications toolbars
			//	This object is required for the toolBars in this file
         // Properties of the toolBars are derived from this

*- Params...appBaseObj, Base Object of the application (_app.myApp)

*- Custom...None

*- Usage....Define as an object of the application base object
			// this.toolBars = new ToolBarBase(this)
         // Define toolbars as an object of the toolbar base
         // this.toolBars.tool1 = new DataToolBar(this.toolBars)
*----------- 
	// Default properties for toolbars
   this.flat  = true
   this.large = true
   this.hints = true
   this.imageHeight = 24

   // Reference to the Application Base object
   this.appBase = appBaseObj

	// Method to change button height
   function SetLarge(lLarge)
		if lLarge
      	this.large = true
      	this.imageHeight = 24
      else
      	this.large = false
			this.imageHeight = 16
      endif
      return

endclass


class FrameToolBar (BaseObj) of Toolbar custom
*-----------
*- Name.....MDI Frame ToolBar (FrameToolBar)
*- Author...Todd Kreuter
*- History..1998.1122 - Original

*- Notes....Written for use with Application Base apps

*- Params...BaseObj, Base Object of the applications toolbars
			// See ToolBarBase Class

*- Custom...None

*- Usage....Define as a property of the toolBar base
			// this.toolBars.tool0 = new frameToolBar(this.toolBars)
*----------- 
   local cBitsize
   cBitsize = IIF( baseObj.large, "TL_", "TS_" )

   this.flat := baseObj.flat
   this.imageHeight = baseObj.imageHeight
   this.text := "Main"
   this.parent = baseObj


   this.tbMenu = new tbMenu(this, cBitsize)
   this.tbMenu.onClick = class::tbMenu_onClick

   function tbMenu_OnClick
   	if type("this.parent.parent.appBase") == "O"
      	local appBase
         appBase = this.parent.parent.appBase
         try
	         appBase.lockDesktop()
				appBase.ABMain.setfocus()
	      catch(exception e)
   	      LockWindowUpdate(0)
      	   msgBox(e.message + " Line: " + ltrim(str(e.lineNo)), "Error", 16)
         finally
	         lockWindowUpdate(0)
         endtry
      endif
	   return

endclass

class DataToolBar (baseObj) of Toolbar custom
*----------- 
*- Name.....Data ToolBar (DataToolBar)
*- Author...Todd Kreuter
*- History..1998.1122 Original
			// 1999.0811 Changes to speed up update routine	

*- Notes....

*- Params...baseObj, See ToolBarBase Class

*- Custom...None

*- Usage....Define as a property of the toolBar base
			// this.toolBars.data = new DataToolBar(this.toolBars)
*----------- 
   local cBitsize
   cBitsize = IIF( baseObj.large, "TL_", "TS_" )
   this.flat := baseObj.flat
   this.imageHeight = baseObj.imageHeight
   this.text := "Data"

	// Define Editing ToolButtons
   this.tbAppend = new tbAppend(this, cBitsize)
   this.tbDelete = new tbDelete(this, cBitsize)
   this.tbSave = new tbSave(this, cBitsize)
   this.tbAbandon = new tbAbandon(this, cBitsize)
	// Define Separator
   this.tbSep1 = new toolbutton(this)
   this.tbSep1.separator = true
   // Define VCR ToolButtons
   this.tbFirst = new tbFirst(this, cBitsize)
   this.tbPrev = new tbPrev(this, cBitsize)
   this.tbNext = new tbNext(this, cBitsize)
   this.tbLast = new tbLast(this, cBitsize)

	// To speed up the update process
   this.lastState = 9

   function onUpdate
   *****************
	local r, nState, lADRow, lSARow, lAtFirst, lAtLast
   // lADRow, Append Delete Row
   // lSARow, Save Abandon Row

	// Check for valid rowset assigned to this.form
	if type("this.form.rowset.state") # "N"
   	return false
	endif

	// Update Editing ToolButtons
	// Update only if rowset.state has changed
	r = this.form.rowset	// Shorten reference
   nState = r.state
   if this.lastState # nState
	   do case
		case nState = 0
	   	with ( this )
				tbAppend.enabled  = false
				tbDelete.enabled  = false
				tbSave.enabled    = false
				tbAbandon.enabled = false
				tbFirst.enabled	= false
				tbPrev.enabled		= false
				tbNext.enabled		= false
				tbLast.enabled		= false
		   endwith
	   	return false
	   case nState = 1
	   	lADRow = true
	      lSARow = false
	   case nState = 2
	   	lADRow = true
   	   lSARow = true
	   case nState = 3
	   	lADRow = false
	      lSARow = true
	   otherwise
	   	lADRow = false
	      lSARow = false
	   endcase

	   with ( this )
			tbAppend.enabled  := lADRow
			tbDelete.enabled  := lADRow
			tbSave.enabled    := lSARow
			tbAbandon.enabled := lSARow
      endwith

	   this.lastState = nState

   endif

   // Update VCR ToolButtons
	if r.endofset
	   lAtFirst	= true
	   lAtLast 	= true
   else
		lAtFirst = r.atFirst()
		lAtLast  = r.atLast()
   endif

   with ( this )
		tbFirst.enabled := ( NOT lAtFirst )
		tbPrev.enabled  := ( NOT lAtFirst )
		tbNext.enabled  := ( NOT lAtLast )
		tbLast.enabled  := ( NOT lAtLast )
   endwith

	return true

endclass

class VCRToolBar (baseObj) of Toolbar
*------------------------------------
*- Name.....VCR ToolBar (VCRToolBar)
*- Author...Todd Kreuter
*- History..1998.1122 Original
*- Notes....

*- Params...baseObj, see ToolBarBase Class

*- Custom...None

*- Usage....Define as a property of the toolBar base
			// this.toolBars.vcr = new vcrToolBar(this.toolBars)
*----------- 
   local cBitsize
   cBitsize = IIF( baseObj.large, "TL_", "TS_" )

   this.flat := baseObj.flat
   this.text     := "VCR"

	// Define toolbuttons
	this.tbFirst = new tbFirst(this, cBitsize)
	this.tbPrev = new tbPrev(this, cBitsize)
	this.tbNext = new tbNext(this, cBitsize)
	this.tbLast = new tbLast(this, cBitsize)

	Function onUpdate
   *****************
	local lAtFirst, lAtLast

	// Check for active query
   if type("this.form.rowset.parent.active") # "L"
	   with ( this )
			tbFirst.enabled	= false
			tbPrev.enabled		= false
			tbNext.enabled		= false
			tbLast.enabled		= false
	   endwith
	   return false
   endif

	if this.form.rowset.endofset
	   lAtFirst = true
	   lAtLast = true
	else
		lAtFirst = this.form.rowset.atFirst()
		lAtLast  = this.form.rowset.atLast()
   endif

	with (this)
		tbFirst.enabled := (NOT lAtFirst)
		tbPrev.enabled  := (NOT lAtFirst)
		tbNext.enabled  := (NOT lAtLast)
		tbLast.enabled  := (NOT lAtLast)
	endwith
   return true

endclass

class ClipToolBar (baseObj) of Toolbar
*-----------
*- Name.....Clip Board ToolBar (ClipToolBar)
*- Author...Todd Kreuter
*- History..1998.1122 Original

*- Notes....

*- Params...baseObj, see ToolBarBase Class

*- Custom...None

*- Usage....Define as a property of the toolBar base
			// this.toolBars.clip = new ClipToolBar(this.toolBars)
*-----------
   local cBitsize
   cBitsize = IIF( baseObj.large, "TL_", "TS_" )

   this.flat := baseObj.flat
   this.text     := "Clipboard"

   this.tbCut = new tbCut(this, cBitsize)
   this.tbCopy = new tbCopy(this, cBitsize)
   this.tbPaste = new tbPaste(this, cBitsize)

   function onUpdate
   *****************
   local lClip, cControl
   lClip = false
   cControl = "this.form.activeControl"
   if ( TYPE(cControl) == "O" )
      lClip := ( TYPE(cControl + ".copy") == "FP" )
      with ( this )
         tbCut.enabled   := TYPE(cControl + ".cut") == "FP"
         tbCopy.enabled  := TYPE(cControl + ".copy") == "FP"
         tbPaste.enabled := TYPE(cControl + ".paste") == "FP"
      endwith
   else
      with (this)
         tbCut.enabled   := false
         tbCopy.enabled  := false
         tbPaste.enabled := false
      endwith
   endif
   return lClip
  
endclass


**************
* Tool Buttons
**************

class tbAppend (parentObj, cBitsize) of toolbutton(parentObj) custom
*-----------
*- Notes....Will check for the forms firstEditObj when appending a row
			// Need to define firstEditObj as a property of the form
*-----------
   with ( this )
		bitmap   := "RESOURCE " + cBitsize + "APPEND"
		speedTip := "Add Row"
   endwith

   Function OnClick
   ****************
   private formObj
   formObj = this.parent.form
	formObj.rowset.beginAppend()
   if type("formObj.firstEditObj") == "O"
   	formObj.firstEditObj.setFocus()
   endif
	return

endclass

class tbDelete (parentObj, cBitsize) of toolbutton(parentObj) custom
*-----------
*- Notes....Will check for the rowsets controlDelete property, and if true
			// Deleting rows needs to be controled by the rowset events.
         // For example, the rowsets canDelete event needs to return true or
         // false depending on a particular circumstance.

			// Need to define controlDelete as a property of the rowset
*-----------
	with ( this )
		bitmap	:= "RESOURCE " + cBitsize + "DELETE"
      speedTip	:= "Delete Row"
   endwith

   Function OnClick
   ****************
   private formObj
   formObj = this.parent.form
	if type("formObj.rowset.controlDelete") # "U"
      return formObj.rowset.delete()
   endif
	local lFirst, lDelete
   lDelete = false
	if ( not formObj.rowset.endOfSet )
		if ( MSGBOX("Delete the current selected row?", "Confirm",4 ) == 6 )
			lFirst  = formObj.rowset.atFirst()
			lDelete = formObj.rowset.delete()       
			if ( not lFirst ) 
				formObj.rowset.next(-1)
			endif
			if ( formObj.rowset.endOfSet )
				formObj.rowset.next()
			endif
		endif
	endif
  	return lDelete

endclass

class tbSave (parentObj, cBitsize) of toolbutton(parentObj) custom
************
   with ( this )
		bitmap   := "RESOURCE " + cBitsize + "SAVE"
		speedTip := "Save Row"
   endwith

   Function OnClick
   ****************
	this.parent.form.rowset.save()
	return

endclass

class tbAbandon (parentObj, cBitsize) of toolbutton(parentObj) custom
***************
   with ( this )
		bitmap   := "RESOURCE " + cBitsize + "ABANDON"
		speedTip := "Abandon"
   endwith

   Function OnClick
	****************
	this.parent.form.rowset.abandon()
   return

endclass


class tbFirst (parentObj, cBitsize) of toolbutton(parentObj) custom
*************
   with ( this )
		bitmap	:= "RESOURCE " + cBitsize + "FIRST"
		speedTip	:= "First Row"
		onClick	:= {||this.parent.form.rowset.first()}
   endwith

endclass

class tbPrev (parentObj, cBitsize) of toolbutton(parentObj) custom
************
   with ( this )
		bitmap	:= "RESOURCE " + cBitsize + "PREV"
		speedTip	:= "Previous Row"
   endwith

   Function onClick
   ****************
	local lPrev
	lPrev = this.parent.form.rowset.next(-1)
	if ( not lPrev )
		this.parent.form.rowset.next()
	endif
	return ( lPrev )

endclass

class tbNext (parentObj, cBitsize) of toolbutton(parentObj) custom
************
   with ( this )
		bitmap	:= "RESOURCE " + cBitsize + "NEXT"
		speedTip	:= "Next Row"
   endwith

	Function onClick
   ****************
	local lNext
	lNext = this.parent.form.rowset.next()
	if ( not lNext )
		this.parent.form.rowset.next(-1)
	endif
   return ( lNext )

endclass

class tbLast (parentObj, cBitsize) of toolbutton(parentObj) custom
************
   with ( this )
		bitmap	:= "RESOURCE " + cBitsize + "LAST"
		speedTip	:= "Last Row"
      onClick	:= {||this.parent.form.rowset.last()}
   endwith

endclass

class tbCut (parentObj, cBitsize) of toolbutton(parentObj) custom
***********
   with ( this )
        bitmap   := "RESOURCE " + cBitsize + "CUT"
        speedTip := "Cut"
        onClick  := {||this.parent.form.activeControl.cut()}
   endwith

endclass

class tbCopy (parentObj, cBitsize) of toolbutton(parentObj) custom
************
   with ( this )  
        bitmap   := "RESOURCE " + cBitsize + "COPY"
        speedTip := "Copy"
        onClick  := {||this.parent.form.activeControl.copy()}
   endwith

endclass

class tbPaste (parentObj, cBitsize) of toolbutton(parentObj) custom
*************
   with ( this )
        bitmap   := "RESOURCE " + cBitsize + "PASTE"
        speedTip := "Paste"
        onClick  := {||this.parent.form.activeControl.paste()}
   endwith

endclass

class tbOpen (parentObj, cBitsize) of toolbutton(parentObj) custom
************
   with ( this )
	   bitmap := "RESOURCE " + cBitsize + "OPEN"
	   speedTip := "Open"
   endwith

endclass

class tbMenu (parentObj, cBitsize) of toolbutton(parentObj) custom
************
	if cBitsize = "TL_"
   	this.bitmap := "RESOURCE #1110"
	else
   	this.bitmap	:= "RESOURCE #622"
   endif
	this.speedTip := "Main Menu"
endclass
